@@ -1,17 +1,17 @@
 # =============================================================================
 #
 # TITLE : nephro - functions to deal with nephrological outcomes
-# AUTHOR: Cristian Pattaro
-# DATE  : Jan 31, 2015
+# AUTHOR: Cristian Pattaro, RyosukeFujii
+# DATE  : May 1, 2022
 #
 # =============================================================================
 
 MDRD4 <- function(creatinine, sex, age, ethnicity, method="IDMS")
-   { 
+{ 
    if (!is.null(creatinine) & !is.null(sex) & !is.null(age) & !is.null(ethnicity))
-      {
+   {
       if (is.null(method)) method <- "other"
-
+      
       creatinine <- as.numeric(creatinine)
       ethnicity <- as.numeric(ethnicity)      
       sex <- as.numeric(sex)
@@ -19,7 +19,7 @@ MDRD4 <- function(creatinine, sex, age, ethnicity, method="IDMS")
       n <- length(creatinine)
       
       if (length(sex) == n & length(age) == n & length(ethnicity) == n)
-         {
+      {
          # Identify missing data and store the index
          idx <- c(1:n)[is.na(creatinine) | is.na(sex) | is.na(age) | is.na(ethnicity)]
          
@@ -39,29 +39,29 @@ MDRD4 <- function(creatinine, sex, age, ethnicity, method="IDMS")
          
          # Output
          eGFR * ifelse(method == 'IDMS', 175, 186)
-         } else
-         stop ("Different number of observations between variables") 
       } else
+         stop ("Different number of observations between variables") 
+   } else
       stop ("Some variables are not defined") 
-   }
+}
 
 MDRD6 <- function(creatinine, sex, age, albumin, BUN, ethnicity, method="IDMS")
-   { 
+{ 
    if (!is.null(creatinine) & !is.null(sex) & !is.null(age) & !is.null(albumin) & !is.null(BUN) & !is.null(ethnicity))
-      {      
+   {      
       if (is.null(method)) method <- "other"
-
+      
       creatinine <- as.numeric(creatinine)
       ethnicity <- as.numeric(ethnicity)      
       albumin <- as.numeric(albumin)
       BUN <- as.numeric(BUN)
       sex <- as.numeric(sex)
       age <- as.numeric(age)
-
+      
       n <- length(creatinine)
       
       if (length(sex) == n & length(age) == n & length(ethnicity) == n & length(albumin) == n & length(BUN) == n )
-         {
+      {
          # Identify missing data and store the index
          idx <- c(1:n)[is.na(creatinine) | is.na(sex) | is.na(age) | is.na(ethnicity) | is.na(BUN) | is.na(albumin)]
          
@@ -72,7 +72,7 @@ MDRD6 <- function(creatinine, sex, age, albumin, BUN, ethnicity, method="IDMS")
          ethnicity[is.na(ethnicity)] <- 10
          albumin[is.na(albumin)] <- 10
          BUN[is.na(BUN)] <- 10
-      
+         
          # MDRD6 formula 
          eGFR <- creatinine^(-0.999)*age^(-0.176)*BUN^(-0.17)*albumin^0.318
          eGFR[sex==0] <- eGFR[sex==0] * 0.762
@@ -83,16 +83,16 @@ MDRD6 <- function(creatinine, sex, age, albumin, BUN, ethnicity, method="IDMS")
          
          # Output
          eGFR * ifelse(method == 'IDMS', 161.5, 170)
-         } else
-         stop ("Different number of observations between variables") 
       } else
+         stop ("Different number of observations between variables") 
+   } else
       stop ("Some variables are not defined") 
-   }
+}
 
 Virga <- function(creatinine, sex, age, wt)
-   {
+{
    if (!is.null(creatinine) & !is.null(sex) & !is.null(age) & !is.null(wt))
-      {
+   {
       creatinine <- as.numeric(creatinine)
       sex <- as.numeric(sex)
       age <- as.numeric(age)
@@ -100,7 +100,7 @@ Virga <- function(creatinine, sex, age, wt)
       n <- length(creatinine)
       
       if (length(sex) == n & length(age) == n & length(wt) == n)
-         {
+      {
          # Identify missing data and store the index
          idx <- c(1:n)[is.na(creatinine) | is.na(sex) | is.na(age) | is.na(wt)]
          
@@ -117,28 +117,27 @@ Virga <- function(creatinine, sex, age, wt)
          
          # Restore missing data at the indexed positions
          eGFR[idx] <- NA
-                  
+         
          # Output
          eGFR
-         } else
-         stop ("Different number of observations between variables") 
       } else
+         stop ("Different number of observations between variables") 
+   } else
       stop ("Some variables are not defined") 
-   }
+}
 
 CKDEpi.creat <- function(creatinine, sex, age, ethnicity)
-   { 
+{ 
    if (!is.null(creatinine) & !is.null(sex) & !is.null(age) & !is.null(ethnicity))
-      {
+   {
       creatinine <- as.numeric(creatinine)
       ethnicity <- as.numeric(ethnicity)      
       sex <- as.numeric(sex)
       age <- as.numeric(age)
-
       n <- length(creatinine)
       
       if (length(sex) == n & length(age) == n & length(ethnicity) == n)
-         {
+      {
          # Identify missing data and store the index
          idx <- c(1:n)[is.na(creatinine) | is.na(sex) | is.na(age) | is.na(ethnicity)]
          
@@ -164,24 +163,62 @@ CKDEpi.creat <- function(creatinine, sex, age, ethnicity)
          
          # Output
          141 * eGFR
-         } else
-         stop ("Different number of observations between variables") 
       } else
+         stop ("Different number of observations between variables") 
+   } else
       stop ("Some variables are not defined") 
-   }
+}
+
+CKDEpi.creat.rf <- function (creatinine, sex, age) 
+{
+   if (!is.null(creatinine) & !is.null(sex) & !is.null(age)) {
+      creatinine <- as.numeric(creatinine)
+      sex <- as.numeric(sex)
+      age <- as.numeric(age)
+      n <- length(creatinine)
+      
+      if (length(sex) == n & length(age) == n)
+      {
+         # Identify missing data and store the index
+         idx <- c(1:n)[is.na(creatinine) | is.na(sex) | is.na(age)]
+         
+         # Replace missing data with fake data to avoid problems with formulas
+         creatinine[is.na(creatinine)] <- 10
+         sex[is.na(sex)] <- 10
+         age[is.na(age)] <- 10
+         
+         # CKD-Epi equation
+         k <- a <- numeric(n)
+         k[sex == 0] <- 0.7
+         k[sex == 1] <- 0.9
+         a[sex == 0] <- -0.241
+         a[sex == 1] <- -0.302
+         one <- rep(1, n)
+         eGFR <- apply(cbind(creatinine/k, one), 1, min, na.rm = T)^a * apply(cbind(creatinine/k, one), 1, max, na.rm = T)^-1.200 * 0.9938^age
+         eGFR[sex == 0] <- eGFR[sex == 0] * 1.012
+         
+         # Restore missing data at the indexed positions
+         eGFR[idx] <- NA
+         
+         # Output
+         142 * eGFR
+      } else
+         stop("Different number of observations between variables")
+   } else 
+      stop("Some variables are not defined")
+}
 
 CKDEpi.cys <- function(cystatin, sex, age)
-   { 
+{ 
    if (!is.null(cystatin) & !is.null(sex) & !is.null(age))
-      {
+   {
       cystatin <- as.numeric(cystatin)
       sex <- as.numeric(sex)
       age <- as.numeric(age)
-
       n <- length(cystatin)
       
       if (length(sex) == n & length(age) == n)
-         {
+      {
          # Identify missing data and store the index
          idx <- c(1:n)[is.na(cystatin) | is.na(sex) | is.na(age)]
          
@@ -201,16 +238,16 @@ CKDEpi.cys <- function(cystatin, sex, age)
          
          # Output
          eGFR
-         } else
-         stop ("Different number of observations between variables") 
       } else
+         stop ("Different number of observations between variables") 
+   } else
       stop ("Some variables are not defined") 
-   }
+}
 
 CKDEpi.creat.cys <- function(creatinine, cystatin, sex, age, ethnicity)
-   {
+{
    if (!is.null(cystatin) & !is.null(cystatin) & !is.null(sex) & !is.null(age) & !is.null(ethnicity))
-      {
+   {
       creatinine <- as.numeric(creatinine)
       cystatin <- as.numeric(cystatin)
       sex <- as.numeric(sex)
@@ -219,7 +256,7 @@ CKDEpi.creat.cys <- function(creatinine, cystatin, sex, age, ethnicity)
       n <- length(creatinine)
       
       if (length(sex) == n & length(age) == n & length(ethnicity) == n & length(cystatin) == n)
-         {
+      {
          # Identify missing data and store the index
          idx <- c(1:n)[is.na(creatinine) | is.na(cystatin) | is.na(sex) | is.na(age) | is.na(ethnicity)]
          
@@ -242,41 +279,83 @@ CKDEpi.creat.cys <- function(creatinine, cystatin, sex, age, ethnicity)
          CR <- cbind(creatinine/k,one)
          CY <- cbind(cystatin/0.8,one)                 
          eGFR <- 135 * apply(CR,1,min,na.rm=T)^a * apply(CR,1,max,na.rm=T)^-0.601 * apply(CY,1,min,na.rm=T)^-0.375 * apply(CY,1,max,na.rm=T)^(-0.711) * 0.995^age * k_sex * k_ethnicity
-
+         
          # Restore missing data at the indexed positions
          eGFR[idx] <- NA
-                  
+         
          # Output
          eGFR
-         } else
-         stop ("Different number of observations between variables") 
       } else
+         stop ("Different number of observations between variables") 
+   } else
       stop ("Some variables are not defined") 
-   }   
-   
-Stevens.cys1 <- function(cystatin)
-   { 
-   if (!is.null(cystatin))
+}   
+
+CKDEpi.creat.cys.rf <- function(creatinine, cystatin, sex, age)
+{
+   if (!is.null(cystatin) & !is.null(cystatin) & !is.null(sex) & !is.null(age))
+   {
+      creatinine <- as.numeric(creatinine)
+      cystatin <- as.numeric(cystatin)
+      sex <- as.numeric(sex)
+      age <- as.numeric(age)
+      n <- length(creatinine)
+      
+      if (length(sex) == n & length(age) == n & length(creatinine) == n & length(cystatin) == n)
       {
+         # Identify missing data and store the index
+         idx <- c(1:n)[is.na(creatinine) | is.na(cystatin) | is.na(sex) | is.na(age)]
+         
+         # Replace missing data with fake data to avoid problems with formulas
+         creatinine[is.na(creatinine)] <- 10
+         cystatin[is.na(cystatin)] <- 10
+         sex[is.na(sex)] <- 10
+         age[is.na(age)] <- 10
+         
+         # CKD-Epi equation
+         a <- b <- c <- numeric(n)
+         a[sex == 0] <- 0.7
+         a[sex == 1] <- 0.9
+         b[sex == 0] <- -0.219
+         b[sex == 1] <- -0.144
+         c <- 0.8
+         one <- rep(1, n)               
+         eGFR <- apply(cbind(creatinine/a, one), 1, min, na.rm = T)^b * apply(cbind(creatinine/a, one), 1, max, na.rm = T)^-0.544 * apply(cbind(cystatin/c, one), 1, min, na.rm = T)^-0.323 * apply(cbind(cystatin/c, one), 1, max, na.rm = T)^-0.778 * 0.9961^age
+         eGFR[sex == 0] <- eGFR[sex == 0] * 0.963
+         
+         # Restore missing data at the indexed positions
+         eGFR[idx] <- NA
+         
+         # Output
+         135 * eGFR
+      } else
+         stop ("Different number of observations between variables") 
+   } else
+      stop ("Some variables are not defined") 
+}
+
+Stevens.cys1 <- function(cystatin)
+{ 
+   if (!is.null(cystatin))
+   {
       cystatin <- as.numeric(cystatin)
       76.7*cystatin^(-1.19)
-      } else
+   } else
       stop ("Variable not defined") 
-   }
+}
 
 Stevens.cys2 <- function(cystatin, sex, age, ethnicity)
-   { 
+{ 
    if (!is.null(cystatin) & !is.null(sex) & !is.null(age) & !is.null(ethnicity))
-      {
+   {
       cystatin <- as.numeric(cystatin)
       ethnicity <- as.numeric(ethnicity)
       sex <- as.numeric(sex)
       age <- as.numeric(age)
-      
       n <- length(cystatin)
-
+      
       if (length(sex) == n & length(age) == n & length(ethnicity) == n)
-         {
+      {
          # Identify missing data and store the index
          idx <- c(1:n)[is.na(cystatin) | is.na(sex) | is.na(age) | is.na(ethnicity)]
          
@@ -290,31 +369,31 @@ Stevens.cys2 <- function(cystatin, sex, age, ethnicity)
          eGFR <- cystatin^(-1.17) * age^(-0.13)
          eGFR[sex == 0] <- eGFR[sex == 0] * 0.91
          eGFR[ethnicity == 1] <- eGFR[ethnicity == 1] * 1.06
-
+         
          # Restore missing data at the indexed positions
          eGFR[idx] <- NA
-                  
+         
          # Output
          127.7 *eGFR
-         } else
-         stop ("Different number of observations between variables") 
       } else
+         stop ("Different number of observations between variables") 
+   } else
       stop ("Some variables are not defined") 
-   }
+}
 
 Stevens.creat.cys <- function(creatinine, cystatin, sex, age, ethnicity)
-   { 
+{ 
    if (!is.null(cystatin) & !is.null(sex) & !is.null(age) & !is.null(ethnicity))
-      {
+   {
       creatinine <- as.numeric(creatinine)
       cystatin <- as.numeric(cystatin)
       ethnicity <- as.numeric(ethnicity)
       sex <- as.numeric(sex)
       age <- as.numeric(age)
       n <- length(creatinine)
-
+      
       if (length(sex) == n & length(age) == n & length(ethnicity) == n & length(cystatin) == n)
-         {
+      {
          # Identify missing data and store the index
          idx <- c(1:n)[is.na(creatinine) | is.na(cystatin) | is.na(sex) | is.na(age) | is.na(ethnicity)]
          
@@ -329,22 +408,22 @@ Stevens.creat.cys <- function(creatinine, cystatin, sex, age, ethnicity)
          eGFR <- creatinine^(-0.65) * cystatin^(-0.57) * age^(-0.20)
          eGFR[sex == 0] <- eGFR[sex == 0] * 0.82
          eGFR[ethnicity == 1] <- eGFR[ethnicity == 1] * 1.11
-
+         
          # Restore missing data at the indexed positions
          eGFR[idx] <- NA
-                  
+         
          # Output
          177.6 * eGFR    
-         } else
-         stop ("Different number of observations between variables") 
       } else
+         stop ("Different number of observations between variables") 
+   } else
       stop ("Some variables are not defined") 
-   }
+}
 
 CG <- function(creatinine, sex, age, wt)
-   { 
+{ 
    if (!is.null(creatinine) & !is.null(sex) & !is.null(age) & !is.null(wt))
-      {
+   {
       creatinine <- as.numeric(creatinine)
       wt  <- as.numeric(wt)      
       sex <- as.numeric(sex)
@@ -352,7 +431,7 @@ CG <- function(creatinine, sex, age, wt)
       n <- length(creatinine)
       
       if (length(sex) == n & length(age) == n & length(wt) == n)
-         {
+      {
          # Identify missing data and store the index
          idx <- c(1:n)[is.na(creatinine) | is.na(sex) | is.na(age) | is.na(wt)]
          
@@ -371,8 +450,8 @@ CG <- function(creatinine, sex, age, wt)
          
          # Output
          eGFR
-         } else
-         stop ("Different number of observations between variables") 
       } else
+         stop ("Different number of observations between variables") 
+   } else
       stop ("Some variables are not defined") 
-   }
+}
\ No newline at end of file
